"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const mongodb_1 = require("mongodb");
const Database_1 = require("./Database");
const helpers_1 = require("./helpers");
const logger_1 = require("./logger");
class DatabaseConnector {
    constructor(client) {
        this.client = client;
        this.getDbConnectionUri = ({ host, port, name, protocol }) => {
            return `${protocol}://${host}:${port}/${name}`;
        };
    }
    connect(dbConfig, reconnectTimeout = DatabaseConnector.DEFAULT_RECONNECT_TIMEOUT) {
        return __awaiter(this, void 0, void 0, function* () {
            const uri = this.getDbConnectionUri(dbConfig);
            logger_1.log(`Connecting to ${uri}...`);
            let client;
            do {
                try {
                    client = yield mongodb_1.MongoClient.connect(uri, { ignoreUndefined: true });
                }
                catch (err) {
                    if (!err.message.includes(DatabaseConnector.CONNECTION_REFUSED_ERROR_CODE)) {
                        throw err;
                    }
                    logger_1.log(`${err.message} Retrying...`);
                    yield helpers_1.sleep(reconnectTimeout);
                }
            } while (!client);
            logger_1.log('Connection with database established.');
            this.client = client;
            const db = client.db(dbConfig.name);
            return new Database_1.Database(db);
        });
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            logger_1.log('Closing connection...');
            if (!this.client) {
                return;
            }
            yield this.client.close();
        });
    }
}
DatabaseConnector.CONNECTION_REFUSED_ERROR_CODE = 'ECONNREFUSED';
DatabaseConnector.DEFAULT_RECONNECT_TIMEOUT = 500;
exports.DatabaseConnector = DatabaseConnector;
//# sourceMappingURL=DatabaseConnector.js.map